/**************************************************************************************************
 * Project name:
     Jabuka.mcpar
 * Generated by:
     MicroC PRO for ARM v.6.0.0
 * Date of creation
     7/12/2018
 * Time of creation
     11:11:00 AM
 * Test configuration:
     MCU:             STM32F407VGT6
     Board:           EasyMx PRO v7 for STM32
     Oscillator:      25 MHz external crystal
     MPU Frequency:   120 MHz from PLL
     Compiler:        mikroC PRO for ARM Version 6.0.0
  * MikroE Libraries: ADC, Conversions, C_Stdlib, C_String, C_Type, Mmc, PWM, SDIO, SPI, Sprintf,
                      TFT, TFT_Defs, TFT_TouchPanel, USB
  * FAT32Library:     FAT32_Defs_STM32_M3_M4_M7, FAT32_Driver_STM32_M3_M4_M7, FAT_32_M3_M4_M7

  This is a test of the FreeRTOS software timers. The demo version of FreeRTOSConfig.h file
  does not have timers enabled. It must be modified with the following declarations:
  #define configUSE_TIMERS                  1
  #define configTIMER_TASK_PRIORITY         10
  #define configTIMER_QUEUE_LENGTH          5
  #define configTIMER_TASK_STACK_DEPTH      5
  #define configSUPPORT_DYNAMIC_ALLOCATION  1

  To enable USB, set SW12 bits 3 (TX), 4 (RX), and 7 (VBUS) to the right. USB UART A interferes
  is incompatible with USB and therefore, must be disabled with SW12 bits 1, and 2 moved to the
  left. Use HID terminal from the Tools menu and set Append CR and Append LF check-marks on. After
  powering-up the board, it takes about 11 seconds for USB to enumerate. You should see Jabuka in
  the HID Devices box. Select it and then enter 't' in the Communications box. Press Send button.
  The box below should display 2 messages 0.5 seconds apart with each press.
  One-shot timer started
  One-shot time now: 0011EDD3
  One-shot timer started
  One-shot time now: 0011F36B
  One-shot timer started
  One-shot time now: 0011F887

  If using TFT display and Touch Panel, do the following.
  Make a separate project in a sub-directory. See TFT Proj in this one. Use the scheme provided
  Develop your screens and
  debug them in that directory. Then start the FreeRTOS project and add the following sources:
  TFT-driver.c
  TFT_events_code.c
  TFT_main.c
  TFT_resources.c
  Click on Project, Edit Search Paths... and add the TFT Project path to the Headers Path window.
  Change the void main(void) statement to void tftTask(void *pvParameters) in TFT_main.c file.
  Make sure you comment out the TFT_Init(320, 240); statement in TFT_diver.c file under
  static void InitializeTouchPanel() subroutine. That is a bug in the compiler that I couldn't
  get rid of.
  
**************************************************************************************************/

// Include files
#include <stdint.h>
#include <stddef.h>
#include "FreeRTOSConfig.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "timers.h"
#include "projdefs.h"
#include "portmacro.h"

#include "main.h"
#include "Init_task.h"
#include "MyTimers.h"
#include "UART2_driver.h"


// Variables




/**************************************************************************************************
  Main function, which creates the tasks and starts the scheduler
**************************************************************************************************/
void main() {

     // Create a FreeRTOS one-shot timer. The code is in MyTimers.c
     CreateOneShotTimer();    // Only works if done here. Has to be started explicitly.
     
     // Create a FreeRTOS auto-reload timer. The code is in MyTimers.c
     CreateAutoReloadTimer(); // Has to be started explicitly.

     
     // Create init task
     xTaskCreate (
          (TaskFunction_t)initTask,
          "Init Task",
          configMINIMAL_STACK_SIZE,
          NULL,
          10,
          NULL
     );

    // Start the RTOS scheduler
    vTaskStartScheduler();

    // Will never reach here
    while (1);
}