/**************************************************************************************************
  MyTimers.h

  Copyright (C) 2015  Val Gretchev

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************************************/
#ifndef MY_TIMERS_H
#define MY_TIMERS_H

// Defines
#define NUMTIMERS 15 // Increase if more timer required

// Structures
struct SoftwareTimers {
     unsigned long int TimerField;
     unsigned short int Free;
};


// Function Prototypes
void Timer_Initialize(void);
void TimerCheck();



/**************************************************************************************************
unction:   init_timerpool
Parameters: none
Return val: void
Desc:       Initialize timer pool
**************************************************************************************************/
void init_timerpool(void);


/**************************************************************************************************
Function:   get_timer
Parameters: none
Return val: unsigned short int - returns handle of free timer
Desc:       Gets free timer from the timer pool
**************************************************************************************************/
unsigned short int get_timer(void);


/**************************************************************************************************
Function:   free_timer
Parameters: none
Return val: unsigned short int nbr - number
Desc:       Puts the given timer back to the timer pool
**************************************************************************************************/
void free_timer(unsigned short int nbr);


/**************************************************************************************************
Function:   set_timer
Parameters: unsigned short int nbr - number
            unsigned long int tout - time out
Return val: none
Desc:       Initializes given timer to given time-out
**************************************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout);


/**************************************************************************************************
Function:   check_timer
Parameters: unsigned short int nbr - number
Return val: unsigned long int - returns the value of given timer
Desc:       Return the value of given timer
**************************************************************************************************/
unsigned long int check_timer(unsigned short int nbr);


/**************************************************************************************************
Function:   irqhandler_timer
Parameters: void
Return val: void
Desc:       Call this function on timer interrupt
**************************************************************************************************/
void irqhandler_timer(void);






/**************************************************************************************************
  Timer 1 Channel 1 used for BLED PWM
**************************************************************************************************/
void Timer1_Initialize(void);
void PwmUp(void);
void PwmDn(void);
void PwmMaxOn(void);
void PwmMaxOff(void);

/**************************************************************************************************
  Timer 3 is used as a single-shot pulse on PC6
**************************************************************************************************/
void Timer3_Initialize(void);
void Timer3_Ch1_Start(void);
void Timer3_Ch1_Stop(void);
void Timer3_Update(void);

/**************************************************************************************************
  Timer 4 is used as a single-shot pulse on PD12
**************************************************************************************************/
void Timer4_Initialize(void);
void Timer4_Ch1_Start(void);

/**************************************************************************************************
  Timer 5
**************************************************************************************************/
void Timer5_Initialize(void);
void Timer5Start(void);
void Timer5Stop(void);

/**************************************************************************************************
  Timer 6
**************************************************************************************************/
void Timer6_Initialize(void);
void Timer6Start(void);
void Timer6Stop(void);

/**************************************************************************************************
  Timer 7
**************************************************************************************************/
void Timer7_Initialize(void);
void Timer7Start(void);
void Timer7Stop(void);

/**************************************************************************************************
  Timer 8 is used as a single-shot pulse on PC7
**************************************************************************************************/
void Timer8_Initialize(void);
void Timer8_Ch2_Start(void);

/**************************************************************************************************
  Timer 10
**************************************************************************************************/
void Timer10_Initialize(void);
void BuzzerOn(void);
void BuzzerOff(void);
void TimeBuzzer(unsigned Msec);

/**************************************************************************************************
  Timer 11
**************************************************************************************************/
void Timer11_Initialize(void);
void Timer11_Start(void);
void Timer11_Stop(void);
void Timer11_Update(void);

// Diagnostics
void DisplayTIM3Regs(void);
void DisplayTIM4Regs(void);
void DisplayTIM8Regs(void);
void DisplayGPIOCRegs(void);

// FreeRTOS one-shot software timer
void prvOneShotTimerCallback(TimerHandle_t xTimer);
void CreateOneShotTimer(void);
void StartOneShotTimer(void);
void ChangeTimerPeriod(TickType_t Period);

// FreeRTOS auto-reload software timer
void SetAutoReloadOutput(void);
void prvAutoReloadTimerCallback(TimerHandle_t xTimer);
void CreateAutoReloadTimer(void);
void StartAutoReloadTimer(void);
void StopAutoReloadTimer(void);
void ChangeReloadTimerPeriod(TickType_t Period);

// Diagnostics
void IndOn(void);
void IndOff(void);
void IndTog(void);









#endif



/**************************************************************************************************
  End of File
**************************************************************************************************/